#include <stdio.h>
#include "inputArm.h"
#include "servoArm.h"
#include "transformer.h"
#include "returnValues.h"
#include <stdint.h>

#include "colours.h"

void setupTerm() {
    int character;

    struct termios new_term_attr;

    /* set the terminal to raw mode */
    tcgetattr(fileno(stdin), &new_term_attr);
    new_term_attr.c_lflag &= ~(ECHO|ICANON);
    new_term_attr.c_cc[VTIME] = 0;
    new_term_attr.c_cc[VMIN] = 0;
    tcsetattr(fileno(stdin), TCSANOW, &new_term_attr);

    /* read a character from the stdin stream without blocking */
    /*   returns EOF (-1) if no character is available */
   // character = fgetc(stdin);

   // return character;

}


void targetingColour( float this, float that) {
	 if (0) {
	 } else if ((this - that) > 4) {
	   textcolor(CC_RED);
	 } else if ((this - that) < -4) {
	   textcolor(CC_BLUE);
	 } else if ((this - that) > 2) {
	   textcolor(CC_YELLOW);
	 } else if ((this - that) < -2) {
	   textcolor(CC_YELLOW);  	   
	 } else {
	   textcolor(CC_GREEN);
	 } 
}

void selectColour( char this, char that) {
  	 if (this == that) {
	   textbackground(CC_YELLOW); 
	 } else {
	   textbackground(CC_BLACK);
	 } 
}


void addDisplayHistory(float * data) {
  static unsigned int count = 0;
  static float Buffer[4][6];
  unsigned int i, j;
  
  for(i = 3; i >0; i--) {
    for(j = 0; j < 6; j++) Buffer[i][j] = Buffer[i-1][j];      
  }
  
  for(j = 0; j < 6; j++) Buffer[0][j] = data[j]; 
  
  for(i = 0; i < 4; i++)  printf(" %d:  %-7.2f, %-7.2f, %-7.2f, %-7.2f, %-7.2f, %-7.2f              \n", count-i, Buffer[i][0], Buffer[i][1], Buffer[i][2], Buffer[i][3], Buffer[i][4], Buffer[i][5]);
  count++;

}


int main(int argc, char * argv[]) {

   servoArm_t    OUT, OUT2;
   armTrainer_t  INP;
   transformSet_t BRIDGE;
   float last[6];
   unsigned int  i;
   unsigned char operate;
   FILE *save;
   char c, pause = 1, select = 0, realtime = 0;
   unsigned long long t, lt;
   struct timeval tv;
   
   
   if (argc < 5) {
     printf("Usage %s devicein configin deviceout configout\n", argv[0]);
     return 0;
   }   
   
   printf("Initializing\n");
   if (armTrainerInit(&INP, argv[1], argv[2]) != OK) {
     printf("port open error\n");
     return 0;
   }
   if (servoArmInit(&OUT, argv[3], argv[4]) != OK) {
     printf("port open error\n");
     return 0;
   }
   
   if ((save = fopen("saved.motion", "wt")) == NULL) {
     printf("Cannot open save out file\n");
     return 0;
   }
   
   transZeroInput(&(OUT.buffer)) ; 
   clrscr();
            
   if (transformSetInit(&BRIDGE, 1) != OK) {
     printf("bridge init err\n");
     return 0;
   }

   if ( transformSetLoadFrom(&BRIDGE, "bridge.tf") != OK) {
     printf("bridge init err, does bridge.tf exist?\n");
     return 0;
   }
   gotoxy(1,1);
   printf(" Space or s to save position to file\n");
   printf(" p to toggle pause\n");
   printf(" r to toggle realtime\n");
   printf(" q to quit\n");
   printf(" = during pause to reset offsets\n");
   printf(" arrows during pause to adust position\n");
   
   setupTerm();
       
   operate = 1;
   while(operate) {               
   
     transCalc(&(INP.buffer));
     
     for (i = 0; i < OUT.buffer.count; i++)  OUT.buffer.in[i] = 0;
     
     /*
     
      [in INP out]  [in BRIDGE out] [in OUT out]
      ADC     DEG    DEG       DEG   DEG    SERVO_COUNT
     
     */
     
     // force a calc between the ends of the two adjacent buffers
     transformSetDo(&BRIDGE, INP.buffer.out, OUT.buffer.in);
     
     transCalc(&(OUT.buffer));
     
     if (!pause)  {
       servoArmSync(&OUT); 
       last[0] = OUT.buffer.in[0]; last[1] = OUT.buffer.in[1]; last[2] = OUT.buffer.in[2]; last[3] = OUT.buffer.in[3]; last[4] = OUT.buffer.in[4];  last[5] = OUT.buffer.in[5];
           
     } else {
         gotoxy(8, 1);
	 textcolor(CC_WHITE);
	 printf("-- PAUSED --\n");
	 	 
         selectColour(select, 0); printf(" %-7.2f,",  last[0]);  
	 selectColour(select, 1); printf(" %-7.2f,",  last[1]);  
	 selectColour(select, 2); printf(" %-7.2f,",  last[2]);  
	 selectColour(select, 3); printf(" %-7.2f,",  last[3]); 
	 selectColour(select, 4); printf(" %-7.2f,",  last[4]);  
	 selectColour(select, 5); printf(" %-7.2f\n", last[5]); 
	 selectColour(1, 2);	 
	   
	// printf(" %-7.2f, %-7.2f, %-7.2f, %-7.2f, %-7.2f, %-7.2f              \n", last[0], last[1], last[2], last[3], last[4], last[5]);	 	 
	 
	 targetingColour( OUT.buffer.in[0], last[0]);  printf(" %-7.2f,", OUT.buffer.in[0]);	 
	 targetingColour( OUT.buffer.in[1], last[1]);  printf(" %-7.2f,", OUT.buffer.in[1]);	 
	 targetingColour( OUT.buffer.in[2], last[2]);  printf(" %-7.2f,", OUT.buffer.in[2]);
	 targetingColour( OUT.buffer.in[3], last[3]);  printf(" %-7.2f,", OUT.buffer.in[3]);
	 targetingColour( OUT.buffer.in[4], last[4]);  printf(" %-7.2f,", OUT.buffer.in[4]);
	 targetingColour( OUT.buffer.in[5], last[5]);  printf(" %-7.2f",  OUT.buffer.in[5]);
	 printf("           \r");	 
	 
     }
     
    // gotoxy(2,1);
     
     //transDump(&(INP.buffer));         
     
     if ((c = fgetc(stdin)) != (-10)) {
       switch (c) {
         case 's':
	 case ' ':
	   gotoxy(20,1); textcolor(CC_WHITE);
	   printf("------------------------| FILE |----------------------------\n");
	   addDisplayHistory(last);
	   
	   if (!realtime) {
	     fprintf(save, "1, %f, %f, %f, %f, %f, %f\n", last[0], last[1], last[2], last[3], last[4], last[5]);
	   } else {
	     /*  gettimeofday(&tv,NULL);
	       t = tv.tv_sec*(uint64_t)1000000+tv.tv_usec;
	       fprintf(save, "%f, %f, %f, %f, %f, %f, %f\n", (float)(t-lt)/(float)1000000, last[0], last[1], last[2], last[3], last[4], last[5]);
	       lt = t;
	       usleep(60000); 
	       fflush(stdin);*/
	   }
	   
	   fflush(save);
	 break;
	 
	 case 'q':
	   operate = 0;
	 break;
	 
	 case 'r':
	    realtime = 1 - realtime;
	    gotoxy(11,1);
	    if (realtime) {
	      printf("realtime mode.\n");
	    } else {
	      printf("1s points mode.\n");
	    }
	 break;
	 
	 case 'p':
	   pause = 1 - pause;
	   if (!pause) 	lt = tv.tv_sec*(uint64_t)1000000+tv.tv_usec;   
	   
	 break;
	 
	 case '=':
	   if (!pause) break;
	   
	   gotoxy(11,1);
	   BRIDGE.transform[0].translate  =  last[0] - INP.buffer.out[0] ;
	   BRIDGE.transform[1].translate  =  last[1] - INP.buffer.out[1] ;	
	   BRIDGE.transform[2].translate  =  last[2] - INP.buffer.out[2] ;
	   BRIDGE.transform[3].translate  =  last[3] - INP.buffer.out[3] ;
	   BRIDGE.transform[4].translate  =  last[4] - INP.buffer.out[4] ;
	   BRIDGE.transform[5].translate  =  last[5] - INP.buffer.out[5] ;
	
	   printf("offsets applied.\n");
	   
	 break;
	 
	 // cursor control
	 case '\e':
	   gotoxy(19,1);
	   if ((c = fgetc(stdin)) == 0x5B) {	   
	     c = fgetc(stdin);
	     if (0) {
	     } else if (c == 0x41) {
	      last[select] += 0.5;
	     } else if (c == 0x42) {
	      last[select] -= 0.5;;
	     } else if (c == 0x43) {
	      if (select < 5) select++;
	     } else if (c == 0x44) {
	      if (select > 0) select--;
	     }
	                 
           OUT.buffer.in[0] = last[0]; 
	   OUT.buffer.in[1] = last[1]; 
	   OUT.buffer.in[2] = last[2]; 
	   OUT.buffer.in[3] = last[3]; 
	   OUT.buffer.in[4] = last[4];  
	   OUT.buffer.in[5] = last[5];
	   
           transCalc(&(OUT.buffer));
	   servoArmSync(&OUT); 
	     
	   }
	 break;
	 
       }      
     }
      if (!realtime) {
       usleep(20000);
      } else {
       if (!pause) {
         gettimeofday(&tv,NULL);
	 t = tv.tv_sec*(uint64_t)1000000+tv.tv_usec;
	 fprintf(save, "%f, %f, %f, %f, %f, %f, %f\n", (float)(t-lt)/(float)1000000, last[0], last[1], last[2], last[3], last[4], last[5]);
	 lt = t;
	 usleep(60000); 
	 fflush(stdin);    
	 gotoxy(20,1); textcolor(CC_WHITE);
	 printf("------------------------| FILE |----------------------------\n");
	 addDisplayHistory(last);   
       
       }
      }
     
   
   }
   
   fclose(save);
  

  return 0;
}

