#include <stdio.h>
#include "inputArm.h"
#include "servoArm.h"
#include "transformer.h"
#include "returnValues.h"

#include "colours.h"

void setupTerm() {
    int character;

    struct termios new_term_attr;

    /* set the terminal to raw mode */
    tcgetattr(fileno(stdin), &new_term_attr);
    new_term_attr.c_lflag &= ~(ECHO|ICANON);
    new_term_attr.c_cc[VTIME] = 0;
    new_term_attr.c_cc[VMIN] = 0;
    tcsetattr(fileno(stdin), TCSANOW, &new_term_attr);

    /* read a character from the stdin stream without blocking */
    /*   returns EOF (-1) if no character is available */
   // character = fgetc(stdin);

   // return character;

}

int main(int argc, char * argv[]) {

   servoArm_t    OUT, OUT2;
   armTrainer_t  INP;
   transformSet_t BRIDGE;
   unsigned int  i;
   unsigned char operate;
   FILE *save;
   char c;
   
   if (argc < 5) {
     printf("Usage %s devicein configin deviceout configout\n", argv[0]);
     return 0;
   }   
   
   printf("Initializing\n");
   if (armTrainerInit(&INP, argv[1], argv[2]) != OK) {
     printf("port open error\n");
     return 0;
   }
   if (servoArmInit(&OUT, argv[3], argv[4]) != OK) {
     printf("port open error\n");
     return 0;
   }
   
   if ((save = fopen("saved.motion", "wt")) == NULL) {
     printf("Cannot open save out file\n");
     return 0;
   }
   
   transZeroInput(&(OUT.buffer)) ; 
   clrscr();
            
   if (transformSetInit(&BRIDGE, 1) != OK) {
     printf("bridge init err\n");
     return 0;
   }

   if ( transformSetLoadFrom(&BRIDGE, "bridge.tf") != OK) {
     printf("bridge init err\n");
     return 0;
   }
   
   setupTerm();
       
   operate = 1;
   while(operate) {               
   
     transCalc(&(INP.buffer));
     
     for (i = 0; i < OUT.buffer.count; i++)  OUT.buffer.in[i] = 0;
     
     transformSetDo(&BRIDGE, INP.buffer.out, OUT.buffer.in);
     
     transCalc(&(OUT.buffer));
     servoArmSync(&OUT); 
     
     gotoxy(2,1);
     
     //transDump(&(INP.buffer));         
     
     if ((c = fgetc(stdin)) != (-10)) {
       switch (c) {
         case 's':
	 case ' ':
	   fprintf(save, "1, %f, %f, %f, %f, %f, %f\n", OUT.buffer.in[0], OUT.buffer.in[1], OUT.buffer.in[2], OUT.buffer.in[3], OUT.buffer.in[4], OUT.buffer.in[5]);
	 break;
	 
	 case'q':
	   operate = 0;
	 break;
       }      
     }
      
       usleep(20000);
     
   
   }
   
   fclose(save);
  

  return 0;
}

